@@ -1,16 +1,69 @@
 import asyncio
 import concurrent.futures
 import contextlib
+import contextvars
 import enum
+import socket
+import ssl
+import sys
 import threading
 import warnings
 import weakref
+from typing import (
+    IO,
+    Any,
+    Awaitable,
+    Callable,
+    Coroutine,
+    Dict,
+    Generator,
+    List,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    TypeVar,
+    Union,
+    cast,
+)
 
 from ._handle import _ProxyHandle, _ProxyTimerHandle
 from ._protocol import _BaseProtocolProxy, _proto_proxy, _proto_proxy_factory
 from ._server import _ServerProxy
 from ._task import Future, Task
-from ._transport import _make_transport_proxy
+from ._transport import _BaseTransportProxy, _make_transport_proxy
+
+if sys.version_info >= (3, 8):
+    from typing import Protocol
+else:
+    from typing_extensions import Protocol
+
+_R = TypeVar("_R")
+
+# stable
+class _HasFileno(Protocol):
+    def fileno(self) -> int:
+        ...
+
+
+_FileDescriptor = int  # stable
+_FileDescriptorLike = Union[int, _HasFileno]  # stable
+
+_Address = Union[Tuple[str, int], Tuple[str, int, int, int]]
+
+_ExceptionContext = Dict[str, Any]
+_ExceptionHandler = Callable[[asyncio.AbstractEventLoop, _ExceptionContext], Any]
+_ProtocolT = TypeVar("_ProtocolT", bound=asyncio.BaseProtocol)
+_ProtocolFactory = Callable[[], asyncio.BaseProtocol]
+_SSLContext = Union[bool, None, ssl.SSLContext]
+_TransProtPair = Tuple[asyncio.BaseTransport, asyncio.BaseProtocol]
+
+
+def _get_fd(fd: _FileDescriptorLike) -> int:
+    if isinstance(fd, int):
+        return fd
+    else:
+        return fd.fileno()
 
 
 class CheckKind(enum.Flag):
@@ -30,40 +83,45 @@ class LoopProxy(asyncio.AbstractEventLoop):
     # isinstance check is much slower
     _proxy_loop_marker = True
 
-    def __init__(self, parent) -> None:
+    def __init__(self, parent: asyncio.AbstractEventLoop) -> None:
         assert isinstance(parent, asyncio.AbstractEventLoop)
         self._parent = parent
         self._closed = False
 
-        self._ready = set()
-        self._timers = set()
-        self._readers = {}
-        self._writers = {}
-        self._tasks = weakref.WeakSet()
-        self._transports = weakref.WeakSet()
-        self._servers = weakref.WeakSet()
-        self._signals = {}
+        self._ready: Set[_ProxyHandle] = set()
+        self._timers: Set[_ProxyTimerHandle] = set()
+        self._readers: Dict[int, asyncio.Handle] = {}
+        self._writers: Dict[int, asyncio.Handle] = {}
+        self._tasks: "weakref.WeakSet[Task[Any]]" = weakref.WeakSet()
+        self._transports: "weakref.WeakSet[_BaseTransportProxy]" = weakref.WeakSet()
+        self._servers: "weakref.WeakSet[_ServerProxy]" = weakref.WeakSet()
+        self._signals: Dict[int, asyncio.Handle] = {}
 
-        self._task_factory = None
-        self._default_executor = None
+        self._task_factory: Optional[
+            Callable[
+                [asyncio.AbstractEventLoop, Generator[Any, None, _R]],
+                "asyncio.Future[_R]",
+            ]
+        ] = None
+        self._default_executor: Any = None
         self._executor_shutdown_called = False
 
-        self._root_task = None
-        self._time_offset = 0
+        self._root_task: Optional["asyncio.Future[Any]"] = None
+        self._time_offset: float = 0.0
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         running = self.is_running()
         closed = self.is_closed()
         debug = self.get_debug()
         return f"<LoopProxy running={running} closed={closed} debug={debug}>"
 
     # properties
-    @property
-    def slow_callback_duration(self):
+    @property  # type:ignore[override]
+    def slow_callback_duration(self) -> float:  # type:ignore[override]
         return self._parent.slow_callback_duration
 
     @slow_callback_duration.setter
-    def slow_callback_duration(self, value):
+    def slow_callback_duration(self, value: float) -> None:
         self._parent.slow_callback_duration = value
 
     # Proxy-specific API
@@ -75,7 +133,7 @@ class LoopProxy(asyncio.AbstractEventLoop):
         self._time_offset += offset
         parent_loop = self._parent
         for timer in self._timers:
-            parent = timer._parent
+            parent = cast(asyncio.TimerHandle, timer._parent)
             if parent is None or parent.cancelled():
                 # cancelled or executed
                 continue
@@ -86,7 +144,7 @@ class LoopProxy(asyncio.AbstractEventLoop):
 
     async def check_and_shutdown(self, kind: CheckKind = CheckKind.ALL) -> None:
         for task in list(self._tasks):
-            if task is self._root_task:
+            if task is self._root_task:  # type: ignore
                 continue
             if task.done():
                 continue
@@ -178,15 +236,17 @@ class LoopProxy(asyncio.AbstractEventLoop):
         self._check_closed()
         self._parent.run_forever()
 
-    def run_until_complete(self, coro_or_future):
-        async def main():
+    def run_until_complete(
+        self, coro_or_future: Union[Awaitable[_R], Generator[Any, None, _R]]
+    ) -> _R:
+        async def main() -> _R:
             new_task = not asyncio.isfuture(coro_or_future)
-            future = asyncio.ensure_future(coro_or_future, loop=self)
+            future = asyncio.ensure_future(coro_or_future, loop=self)  # type:ignore
             if new_task:
                 # An exception is raised if the future didn't complete,
                 # so there
                 # is no need to log the "destroy pending task" message
-                future._log_destroy_pending = False
+                future._log_destroy_pending = False  # type: ignore[attr-defined]
             waiter = self._parent.create_future()
             self._chain_future(waiter, future)
             if new_task:
@@ -195,20 +255,20 @@ class LoopProxy(asyncio.AbstractEventLoop):
                 ret = await waiter
             finally:
                 self._root_task = None
-            return ret
+            return cast(_R, ret)
 
         return self._parent.run_until_complete(main())
 
-    def stop(self):
+    def stop(self) -> None:
         return self._parent.stop()
 
-    def is_running(self):
+    def is_running(self) -> bool:
         return self._parent.is_running()
 
-    def is_closed(self):
+    def is_closed(self) -> bool:
         return self._closed
 
-    def close(self):
+    def close(self) -> None:
         if self.is_running():
             raise RuntimeError("Cannot close a running event loop")
         self._closed = True
@@ -223,13 +283,13 @@ class LoopProxy(asyncio.AbstractEventLoop):
             self._default_executor = None
             executor.shutdown(wait=False)
 
-    async def shutdown_asyncgens(self):
+    async def shutdown_asyncgens(self) -> None:
         warnings.warn(
             "Only the original loop can shutdown async generators", RuntimeWarning
         )
         return
 
-    async def shutdown_default_executor(self):
+    async def shutdown_default_executor(self) -> None:
         self._executor_shutdown_called = True
         if self._default_executor is None:
             return
@@ -241,7 +301,7 @@ class LoopProxy(asyncio.AbstractEventLoop):
         finally:
             thread.join()
 
-    def _do_shutdown(self, future):
+    def _do_shutdown(self, future: Future[Any]) -> None:
         try:
             self._default_executor.shutdown(wait=True)
             self.call_soon_threadsafe(future.set_result, None)
@@ -252,7 +312,12 @@ class LoopProxy(asyncio.AbstractEventLoop):
 
     # Methods scheduling callbacks.  All these return Handles.
 
-    def call_soon(self, callback, *args, context=None):
+    def call_soon(
+        self,
+        callback: Callable[..., Any],
+        *args: Any,
+        context: Optional[contextvars.Context] = None,
+    ) -> asyncio.Handle:
         self._check_closed()
         handle = _ProxyHandle(callback, args, self, context)
         parent_handle = self._parent.call_soon(handle._run)
@@ -261,7 +326,13 @@ class LoopProxy(asyncio.AbstractEventLoop):
             del handle._source_traceback[-1]
         return handle
 
-    def call_later(self, delay, callback, *args, context=None):
+    def call_later(
+        self,
+        delay: float,
+        callback: Callable[..., Any],
+        *args: Any,
+        context: Optional[contextvars.Context] = None,
+    ) -> asyncio.TimerHandle:
         self._check_closed()
         timer = _ProxyTimerHandle(self.time() + delay, callback, args, self, context)
         parent_timer = self._parent.call_later(delay, timer._run)
@@ -270,7 +341,13 @@ class LoopProxy(asyncio.AbstractEventLoop):
             del timer._source_traceback[-1]
         return timer
 
-    def call_at(self, when, callback, *args, context=None):
+    def call_at(
+        self,
+        when: float,
+        callback: Callable[..., Any],
+        *args: Any,
+        context: Optional[contextvars.Context] = None,
+    ) -> asyncio.TimerHandle:
         self._check_closed()
         timer = _ProxyTimerHandle(when, callback, args, self, context)
         parent_timer = self._parent.call_at(when, timer._run)
@@ -279,23 +356,27 @@ class LoopProxy(asyncio.AbstractEventLoop):
             del timer._source_traceback[-1]
         return timer
 
-    def time(self):
+    def time(self) -> float:
         return self._parent.time() + self._time_offset
 
-    def create_future(self):
-        fut = Future(loop=self)
-        return fut
+    def create_future(self) -> "asyncio.Future[Any]":
+        return Future(loop=self)  # type: ignore[return-value]
 
     # Method scheduling a coroutine object: create a task.
 
-    def create_task(self, coro, *, name=None):
+    def create_task(
+        self,
+        coro: Union[Awaitable[_R], Generator[Any, None, _R]],
+        *,
+        name: Optional[str] = None,
+    ) -> "asyncio.Task[_R]":
         self._check_closed()
         if self._task_factory is None:
             task = Task(coro, loop=self, name=name)
             if task._source_traceback:
                 del task._source_traceback[-1]
         else:
-            task = self._task_factory(self, coro)
+            task = self._task_factory(self, coro)  # type: ignore
             if name is not None:
                 try:
                     set_name = task.set_name
@@ -304,11 +385,16 @@ class LoopProxy(asyncio.AbstractEventLoop):
                 else:
                     set_name(name)
         self._tasks.add(task)
-        return task
+        return task  # type: ignore[return-value]
 
     # Methods for interacting with threads.
 
-    def call_soon_threadsafe(self, callback, *args, context=None):
+    def call_soon_threadsafe(
+        self,
+        callback: Callable[..., Any],
+        *args: Any,
+        context: Optional[contextvars.Context] = None,
+    ) -> asyncio.Handle:
         self._check_closed()
         handle = _ProxyHandle(callback, args, self, context)
         parent_handle = self._parent.call_soon_threadsafe(handle._run)
@@ -317,7 +403,9 @@ class LoopProxy(asyncio.AbstractEventLoop):
             del handle._source_traceback[-1]
         return handle
 
-    def run_in_executor(self, executor, func, *args):
+    def run_in_executor(
+        self, executor: Any, func: Callable[..., _R], *args: Any
+    ) -> "asyncio.Future[_R]":
         self._check_closed()
         if executor is None:
             executor = self._default_executor
@@ -333,89 +421,160 @@ class LoopProxy(asyncio.AbstractEventLoop):
         self._chain_future(fut, parent_fut)
         return fut
 
-    def set_default_executor(self, executor):
+    def set_default_executor(self, executor: Any) -> None:
         if not isinstance(executor, concurrent.futures.ThreadPoolExecutor):
             raise TypeError("executor must be ThreadPoolExecutor instance")
         self._default_executor = executor
 
     # Network I/O methods returning Futures.
 
-    async def getaddrinfo(self, host, port, **kwargs):
+    async def getaddrinfo(  # type: ignore[override]
+        self, host: Optional[str], port: Union[int, str, None], **kwargs: Any
+    ) -> List[
+        Tuple[int, int, int, str, Union[Tuple[str, int], Tuple[str, int, int, int]]]
+    ]:
         self._check_closed()
-        return await self._wrap_async(self._parent.getaddrinfo(host, port, **kwargs))
+        return await self._wrap_async(
+            self._parent.getaddrinfo(  # type: ignore[arg-type]
+                host,
+                port,
+                **kwargs,
+            )
+        )
 
-    async def getnameinfo(self, sockaddr, flags=0):
+    async def getnameinfo(self, sockaddr: _Address, flags: int = 0) -> Tuple[str, str]:
         self._check_closed()
         return await self._wrap_async(self._parent.getnameinfo(sockaddr, flags))
 
-    async def create_connection(self, protocol_factory, host=None, port=None, **kwargs):
+    async def create_connection(
+        self,
+        protocol_factory: Callable[[], _ProtocolT],
+        host: Optional[str] = None,
+        port: Optional[int] = None,
+        **kwargs: Any,
+    ) -> Tuple[asyncio.BaseTransport, _ProtocolT]:
         self._check_closed()
         _, proto = await self._wrap_async(
             self._parent.create_connection(
-                _proto_proxy_factory(protocol_factory, self), host, port, **kwargs
+                _proto_proxy_factory(protocol_factory, self),
+                host,  # type: ignore[arg-type]
+                port,  # type: ignore[arg-type]
+                **kwargs,
             )
         )
-        transp = proto.transport
+        proto_proxy = cast(_BaseProtocolProxy, proto)
+        transp = proto_proxy.transport
+        assert transp is not None
         self._transports.add(transp)
-        return transp, proto.protocol
+        return transp, proto_proxy.protocol  # type: ignore[return-value]
 
-    async def create_server(self, protocol_factory, host=None, port=None, **kwargs):
+    async def create_server(  # type: ignore[override]
+        self,
+        protocol_factory: Callable[[], _ProtocolT],
+        host: Optional[Union[str, Sequence[str]]] = None,
+        port: Optional[int] = None,
+        **kwargs: Any,
+    ) -> asyncio.AbstractServer:
         self._check_closed()
         server = await self._wrap_async(
             self._parent.create_server(
-                _proto_proxy_factory(protocol_factory, self), host, port, **kwargs
+                _proto_proxy_factory(protocol_factory, self),
+                host,
+                port,  # type: ignore[arg-type]
+                **kwargs,
             )
         )
-        server = _ServerProxy(server, self)
-        self._servers.add(server)
-        return server
+        server_proxy = _ServerProxy(server, self)
+        self._servers.add(server_proxy)
+        return server_proxy
 
-    async def sendfile(self, transport, file, offset=0, count=None, *, fallback=True):
+    async def sendfile(
+        self,
+        transport: asyncio.BaseTransport,
+        file: IO[bytes],
+        offset: int = 0,
+        count: Optional[int] = None,
+        *,
+        fallback: bool = True,
+    ) -> int:
         self._check_closed()
         sent_count = await self._wrap_async(
             self._parent.sendfile(
-                transport._orig, file, offset, count, fallback=fallback
+                cast(_BaseTransportProxy, transport)._orig,
+                file,
+                offset,
+                count,
+                fallback=fallback,
             )
         )
         return sent_count
 
-    async def start_tls(self, transport, protocol, sslcontext, **kwargs):
+    async def start_tls(
+        self,
+        transport: asyncio.BaseTransport,
+        protocol: asyncio.BaseProtocol,
+        sslcontext: ssl.SSLContext,
+        **kwargs: Any,
+    ) -> asyncio.BaseTransport:
         self._check_closed()
         proto = _proto_proxy(protocol, self)
         tr = await self._wrap_async(
-            self._parent.start_tls(transport._orig, proto, sslcontext, **kwargs)
+            self._parent.start_tls(
+                cast(_BaseTransportProxy, transport)._orig,
+                proto,
+                sslcontext,
+                **kwargs,
+            )
         )
         transp = _make_transport_proxy(tr, self)
         proto.transport = transp
         self._transports.add(transp)
         return transp
 
-    async def create_unix_connection(self, protocol_factory, path=None, **kwargs):
+    async def create_unix_connection(
+        self,
+        protocol_factory: Callable[[], _ProtocolT],
+        path: Optional[str] = None,
+        **kwargs: Any,
+    ) -> Tuple[asyncio.BaseTransport, _ProtocolT]:
         self._check_closed()
         _, proto = await self._wrap_async(
             self._parent.create_unix_connection(
-                _proto_proxy_factory(protocol_factory, self), path, **kwargs
+                _proto_proxy_factory(protocol_factory, self),
+                path,
+                **kwargs,
             )
         )
-        transp = proto.transport
+        proto_proxy = cast(_BaseProtocolProxy, proto)
+        transp = proto_proxy.transport
+        assert transp is not None
         self._transports.add(transp)
-        return transp, proto.protocol
+        return transp, proto_proxy.protocol  # type: ignore[return-value]
 
-    async def create_unix_server(self, protocol_factory, path=None, **kwargs):
+    async def create_unix_server(  # type: ignore[override]
+        self,
+        protocol_factory: _ProtocolFactory,
+        path: Optional[str] = None,
+        **kwargs: Any,
+    ) -> asyncio.AbstractServer:
         self._check_closed()
         server = await self._wrap_async(
             self._parent.create_unix_server(
-                _proto_proxy_factory(protocol_factory, self), path, **kwargs
+                _proto_proxy_factory(protocol_factory, self),
+                path,
+                **kwargs,
             )
         )
-        server = _ServerProxy(server, self)
-        self._servers.add(server)
-        return server
+        server_proxy = _ServerProxy(server, self)
+        self._servers.add(server_proxy)
+        return server_proxy
 
-    async def connect_accepted_socket(self, protocol_factory, sock, **kwargs):
+    async def connect_accepted_socket(
+        self, protocol_factory: _ProtocolFactory, sock: socket.socket, **kwargs: Any
+    ) -> Tuple[asyncio.BaseTransport, _ProtocolT]:
         self._check_closed()
         _, proto = await self._wrap_async(
-            self._parent.connect_accepted_socket(
+            self._parent.connect_accepted_socket(  # type: ignore[attr-defined]
                 _proto_proxy_factory(protocol_factory, self), sock, **kwargs
             )
         )
@@ -424,8 +583,12 @@ class LoopProxy(asyncio.AbstractEventLoop):
         return transp, proto.protocol
 
     async def create_datagram_endpoint(
-        self, protocol_factory, local_addr=None, remote_addr=None, **kwargs
-    ):
+        self,
+        protocol_factory: Callable[[], _ProtocolT],
+        local_addr: Optional[Tuple[str, int]] = None,
+        remote_addr: Optional[Tuple[str, int]] = None,
+        **kwargs: Any,
+    ) -> Tuple[asyncio.BaseTransport, _ProtocolT]:
         self._check_closed()
         _, proto = await self._wrap_async(
             self._parent.create_datagram_endpoint(
@@ -435,89 +598,114 @@ class LoopProxy(asyncio.AbstractEventLoop):
                 **kwargs,
             )
         )
-        transp = proto.transport
+        proto_proxy = cast(_BaseProtocolProxy, proto)
+        transp = proto_proxy.transport
+        assert transp is not None
         self._transports.add(transp)
-        return transp, proto.protocol
+        return transp, proto_proxy.protocol  # type: ignore[return-value]
 
     # Pipes and subprocesses.
 
-    async def connect_read_pipe(self, protocol_factory, pipe):
+    async def connect_read_pipe(
+        self, protocol_factory: Callable[[], _ProtocolT], pipe: Any
+    ) -> Tuple[asyncio.ReadTransport, _ProtocolT]:
         self._check_closed()
         _, proto = await self._wrap_async(
             self._parent.connect_read_pipe(
                 _proto_proxy_factory(protocol_factory, self), pipe
             )
         )
-        transp = proto.transport
+        proto_proxy = cast(_BaseProtocolProxy, proto)
+        transp = proto_proxy.transport
+        assert transp is not None
         self._transports.add(transp)
-        return transp, proto.protocol
+        return transp, proto_proxy.protocol  # type: ignore[return-value]
 
-    async def connect_write_pipe(self, protocol_factory, pipe):
+    async def connect_write_pipe(
+        self, protocol_factory: Callable[[], _ProtocolT], pipe: Any
+    ) -> Tuple[asyncio.WriteTransport, _ProtocolT]:
         self._check_closed()
         _, proto = await self._wrap_async(
             self._parent.connect_write_pipe(
                 _proto_proxy_factory(protocol_factory, self), pipe
             )
         )
-        transp = proto.transport
+        proto_proxy = cast(_BaseProtocolProxy, proto)
+        transp = proto_proxy.transport
+        assert transp is not None
         self._transports.add(transp)
-        return transp, proto.protocol
+        return transp, proto_proxy.protocol  # type: ignore[return-value]
 
-    async def subprocess_shell(self, protocol_factory, cmd, **kwargs):
+    async def subprocess_shell(
+        self,
+        protocol_factory: Callable[[], _ProtocolT],
+        cmd: Union[bytes, str],
+        **kwargs: Any,
+    ) -> Tuple[asyncio.SubprocessTransport, _ProtocolT]:
         self._check_closed()
         _, proto = await self._wrap_async(
             self._parent.subprocess_shell(
                 _proto_proxy_factory(protocol_factory, self), cmd, **kwargs
             )
         )
-        transp = proto.transport
+        proto_proxy = cast(_BaseProtocolProxy, proto)
+        transp = proto_proxy.transport
+        assert transp is not None
         self._transports.add(transp)
-        return transp, proto.protocol
+        return transp, proto_proxy.protocol  # type: ignore[return-value]
 
-    async def subprocess_exec(self, protocol_factory, *args, **kwargs):
+    async def subprocess_exec(
+        self, protocol_factory: Callable[[], _ProtocolT], *args: Any, **kwargs: Any
+    ) -> Tuple[asyncio.SubprocessTransport, _ProtocolT]:
         self._check_closed()
         _, proto = await self._wrap_async(
             self._parent.subprocess_exec(
                 _proto_proxy_factory(protocol_factory, self), *args, **kwargs
             )
         )
-        transp = proto.transport
+        proto_proxy = cast(_BaseProtocolProxy, proto)
+        transp = proto_proxy.transport
+        assert transp is not None
         self._transports.add(transp)
-        return transp, proto.protocol
+        return transp, proto_proxy.protocol  # type: ignore[return-value]
 
     # Ready-based callback registration methods.
     # The add_*() methods return None.
     # The remove_*() methods return True if something was removed,
     # False if there was nothing to delete.
 
-    def add_reader(self, fd, callback, *args):
+    def add_reader(
+        self, fd: _FileDescriptorLike, callback: Callable[..., Any], *args: Any
+    ) -> None:
         self._check_closed()
         handle = asyncio.Handle(callback, args, self)
         self._parent.add_reader(fd, self._wrap_cb, handle._run)
-        self._readers[fd] = handle
+        self._readers[_get_fd(fd)] = handle
 
-    def remove_reader(self, fd):
+    def remove_reader(self, fd: _FileDescriptorLike) -> bool:  # type:ignore[override]
         if self.is_closed():
             return False
         self._parent.remove_reader(fd)
-        handle = self._readers.pop(fd, None)
+        handle = self._readers.pop(_get_fd(fd), None)
         if handle is not None:
             handle.cancel()
             return True
         else:
             return False
 
-    def add_writer(self, fd, callback, *args):
+    def add_writer(
+        self, fd: _FileDescriptorLike, callback: Callable[..., Any], *args: Any
+    ) -> None:
         self._check_closed()
         handle = asyncio.Handle(callback, args, self)
-        self._parent.add_writer(fd, self._wrap_cb, handle._run)
-        self._writers[fd] = handle
+        self._parent.add_writer(_get_fd(fd), self._wrap_cb, handle._run)
+        self._writers[_get_fd(fd)] = handle
 
-    def remove_writer(self, fd):
+    def remove_writer(self, fd: _FileDescriptorLike) -> bool:  # type:ignore[override]
         if self.is_closed():
             return False
         self._parent.remove_writer(fd)
-        handle = self._writers.pop(fd, None)
+        handle = self._writers.pop(_get_fd(fd), None)
         if handle is not None:
             handle.cancel()
             return True
@@ -526,27 +714,37 @@ class LoopProxy(asyncio.AbstractEventLoop):
 
     # Completion based I/O methods returning Futures.
 
-    async def sock_recv(self, sock, nbytes):
+    async def sock_recv(self, sock: socket.socket, nbytes: int) -> bytes:
         self._check_closed()
         return await self._wrap_async(self._parent.sock_recv(sock, nbytes))
 
-    async def sock_recv_into(self, sock, buf):
+    async def sock_recv_into(self, sock: socket.socket, buf: bytearray) -> int:
         self._check_closed()
         return await self._wrap_async(self._parent.sock_recv_into(sock, buf))
 
-    async def sock_sendall(self, sock, data):
+    async def sock_sendall(self, sock: socket.socket, data: bytes) -> None:
         self._check_closed()
         return await self._wrap_async(self._parent.sock_sendall(sock, data))
 
-    async def sock_connect(self, sock, address):
+    async def sock_connect(
+        self, sock: socket.socket, address: Union[Tuple[Any, ...], str]
+    ) -> None:
         self._check_closed()
         return await self._wrap_async(self._parent.sock_connect(sock, address))
 
-    async def sock_accept(self, sock):
+    async def sock_accept(self, sock: socket.socket) -> Tuple[socket.socket, Any]:
         self._check_closed()
         return await self._wrap_async(self._parent.sock_accept(sock))
 
-    async def sock_sendfile(self, sock, file, offset=0, count=None, *, fallback=None):
+    async def sock_sendfile(
+        self,
+        sock: socket.socket,
+        file: IO[bytes],
+        offset: int = 0,
+        count: Optional[int] = None,
+        *,
+        fallback: Optional[bool] = None,
+    ) -> int:
         self._check_closed()
         return await self._wrap_async(
             self._parent.sock_sendfile(sock, file, offset, count, fallback=fallback)
@@ -554,67 +752,83 @@ class LoopProxy(asyncio.AbstractEventLoop):
 
     # Signal handling.
 
-    def add_signal_handler(self, sig, callback, *args):
+    def add_signal_handler(
+        self, sig: int, callback: Callable[..., Any], *args: Any
+    ) -> None:
         self._check_closed()
         handle = asyncio.Handle(callback, args, self)
         self._parent.add_signal_handler(sig, self._wrap_cb, handle._run)
         self._signals[sig] = handle
 
-    def remove_signal_handler(self, sig):
+    def remove_signal_handler(self, sig: int) -> bool:
         handler = self._signals.pop(sig, None)
         self._parent.remove_signal_handler(sig)
         return handler is not None
 
     # Task factory.
 
-    def set_task_factory(self, factory):
+    def set_task_factory(
+        self,
+        factory: Optional[
+            Callable[
+                [asyncio.AbstractEventLoop, Generator[Any, None, _R]],
+                "asyncio.Future[_R]",
+            ]
+        ],
+    ) -> None:
         if factory is not None and not callable(factory):
             raise TypeError("task factory must be a callable or None")
         self._task_factory = factory
 
-    def get_task_factory(self):
+    def get_task_factory(
+        self,
+    ) -> Optional[
+        Callable[
+            [asyncio.AbstractEventLoop, Generator[Any, None, _R]], "asyncio.Future[_R]"
+        ]
+    ]:
         return self._task_factory
 
     # Error handlers.
 
-    def get_exception_handler(self):
+    def get_exception_handler(self) -> Optional[_ExceptionHandler]:
         return self._parent.get_exception_handler()
 
-    def set_exception_handler(self, handler):
+    def set_exception_handler(self, handler: Optional[_ExceptionHandler]) -> None:
         return self._parent.set_exception_handler(handler)
 
-    def default_exception_handler(self, context):
+    def default_exception_handler(self, context: _ExceptionContext) -> None:
         return self._parent.default_exception_handler(context)
 
-    def call_exception_handler(self, context):
+    def call_exception_handler(self, context: _ExceptionContext) -> None:
         return self._parent.call_exception_handler(context)
 
     # Debug flag management.
 
-    def get_debug(self):
+    def get_debug(self) -> bool:
         return self._parent.get_debug()
 
-    def set_debug(self, enabled):
+    def set_debug(self, enabled: bool) -> None:
         self._parent.set_debug(enabled)
 
     # Inherited
 
-    def _timer_handle_cancelled(self, handle):
+    def _timer_handle_cancelled(self, handle: asyncio.TimerHandle) -> None:
         # Nothing to do, _ProxyTimerHandle.cancel()
         # already cancelled the parent timer
         pass
 
     # Implementation details
 
-    def _check_closed(self):
+    def _check_closed(self) -> None:
         if self._closed:
             raise RuntimeError("Event loop is closed")
 
-    def _check_default_executor(self):
+    def _check_default_executor(self) -> None:
         if self._executor_shutdown_called:
             raise RuntimeError("Executor shutdown has been called")
 
-    def _wrap_cb(self, __func, *args, **kwargs):
+    def _wrap_cb(self, __func: Callable[..., _R], *args: Any, **kwargs: Any) -> _R:
         # Private API calls are OK here
         loop = asyncio._get_running_loop()
         asyncio._set_running_loop(self)
@@ -623,21 +837,27 @@ class LoopProxy(asyncio.AbstractEventLoop):
         finally:
             asyncio._set_running_loop(loop)
 
-    def _wrap_async(self, coro_or_future):
+    def _wrap_async(
+        self, coro_or_future: Union[Awaitable[_R], Coroutine[Any, Any, _R]]
+    ) -> "asyncio.Future[_R]":
         # Private API calls are OK here
         loop = asyncio._get_running_loop()
         assert loop is None or loop is self
         fut = self.create_future()
-        coro_or_future = asyncio.ensure_future(coro_or_future, loop=self._parent)
-        self._chain_future(fut, coro_or_future)
+        inner_fut = asyncio.ensure_future(coro_or_future, loop=self._parent)
+        self._chain_future(fut, inner_fut)
         return fut
 
-    def _chain_future(self, target, source):
-        def _call_check_cancel(target):
+    def _chain_future(
+        self,
+        target: "asyncio.Future[_R]",
+        source: "asyncio.Future[_R]",
+    ) -> None:
+        def _call_check_cancel(target: "asyncio.Future[_R]") -> None:
             if target.cancelled():
                 source.cancel()
 
-        def _call_set_state(source):
+        def _call_set_state(source: "asyncio.Future[_R]") -> None:
             if source.cancelled():
                 target.cancel()
                 return
